#include <gb.h>
#include "font.c"
#include "fontwidth.c"
#include "bkg_map.c"

UBYTE tt_font_height = 11;
UBYTE tt_tile_offset = 1;
UBYTE tt_tile_height = 4;

UBYTE tt_cursor_x;
UBYTE tt_cursor_y;

static const unsigned char cEMPTY[] = 
{ 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}; 
/* an empty line for the background */

void empty_screen (void)
{
  UBYTE temp1;
 
  HIDE_BKG;
  HIDE_SPRITES;

  VBK_REG = 0;

  for (temp1=0; temp1<40; temp1++)
  {
    move_sprite(temp1,0,200);
    set_sprite_prop (temp1,0);
  }

  for (temp1=0; temp1<20; temp1++)
  {
    set_bkg_tiles (0,temp1,20,1,(unsigned char *) &cEMPTY[0]);   
    set_win_tiles (0,temp1,20,1,(unsigned char *) &cEMPTY[0]);   

    VBK_REG = 1;
    set_bkg_tiles (0,temp1,20,1,(unsigned char *) &cEMPTY[0]);   
    set_win_tiles (0,temp1,20,1,(unsigned char *) &cEMPTY[0]);   
    VBK_REG = 0;
  }

  move_bkg (0,0);
  
  SHOW_BKG;
  SHOW_SPRITES;
  DISPLAY_ON;
}

void tt_empty_line ( UBYTE y )
{
  UBYTE x;
  for (x = y*20 ; x < (y+1)*20 ; x++ )
  {
    set_bkg_data (x+tt_tile_offset,1,(unsigned char *)cEMPTY);
  }
}

void tt_empty_screen ( void )
{
  UBYTE x;
  for ( x = 0 ; x < tt_tile_height ; x++ )
  {
    tt_empty_line ( x );
  }
}  

/*
  Initialize the truetext library.
*/
void tt_init ( void )
{
  tt_cursor_x = 1;
  tt_cursor_y = 1;
}

/*
  Set the cursor to a given position within the text window
*/
void tt_set_cursor (UBYTE x, UBYTE y)
{
  tt_cursor_x = x;
  tt_cursor_y = x;
}

/* 
  tt_size return the pixel length of the word or string passed 
  returns size (maximum of 255)

  0 for size of word
  1 for size of string
*/
UBYTE tt_size (UBYTE word_or_str, char* str)
{
  UBYTE size;
  UBYTE stop;

  size = 0; 
  stop = 1;

  if ((*str == 0) || ((word_or_str == 0) && (*str == ' ')) )
    stop = 0;
  while (stop== 1)
  {
    size += fontwidth[(UBYTE) (* str)];
    str++;
    if ((*str == 0) || ((word_or_str == 0) && (*str == ' ')) )
      stop = 0;
  }
  return size;
}

void tt_draw_segment (UBYTE tile_num, UBYTE tile_x, UBYTE tile_y,
                       UBYTE data_x, UBYTE data_y, UBYTE char_num)
{
  UWORD data;
  UBYTE tiledata [16];
  UBYTE linedata;
  UBYTE temp1;

  data = tile_num;
  data = data << 4;
  data = data + 0x9000; // this won't work with negative tiles
  get_data((unsigned char *) tiledata, (unsigned char *) data, 16);

  data = char_num;
  data = data << 4;

  data = data + data_y + data_y;

  for (temp1 = tile_y + tile_y ; temp1 < 16-data_y-data_y; temp1++)
  {
    linedata = font [data];
    linedata = linedata << data_x;
    linedata = linedata >> tile_x;
    tiledata [temp1] += linedata;
    data ++;
  }

  set_bkg_data (tile_num,1,(unsigned char *)tiledata);
}

void tt_print_letter (UBYTE letter)
{
  UBYTE x_tile;
  UBYTE y_tile;
  
  x_tile = tt_cursor_x >> 3;
  y_tile = tt_cursor_y >> 3;

  tt_draw_segment (x_tile+(y_tile*20)+tt_tile_offset, tt_cursor_x&7, tt_cursor_y&7, 0, 0, letter);
  if (tt_cursor_x & 7)
  {
    tt_draw_segment (x_tile+(y_tile*20)+tt_tile_offset+1, 0, tt_cursor_y&7, 8-(tt_cursor_x&7), 0, letter);
  }
  if (tt_cursor_y & 7)
  {
    tt_draw_segment (x_tile+(y_tile*20)+20+tt_tile_offset, tt_cursor_x&7, 0, 0, 8-(tt_cursor_y&7), letter);
  }
  if ((tt_cursor_y & 7) && (tt_cursor_x & 7))
  {
    tt_draw_segment (x_tile+(y_tile*20)+21+tt_tile_offset, 0, 0, 8-(tt_cursor_x&7), 8-(tt_cursor_y&7), letter);
  }

}

void tt_print_str (char * str)
{
  char * t1;
  UBYTE stop;

  while ((*str) != 0)
  {
    stop = 1;
    if (tt_cursor_x + tt_size (0,str) > 159)
    {
      tt_cursor_x = 1;
      tt_cursor_y += tt_font_height;
    }
    while (stop == 1) 
    {
      if (*str != 0)
      {
        tt_print_letter ((UBYTE) (*str));
        tt_cursor_x += fontwidth [(UBYTE) *str] +1 ;
      }
      if ((*str == 0) || (*str == ' '))
        stop = 0;
      if (*str != 0)
        str++;
    }
  }
  tt_cursor_x = 1;
  tt_cursor_y += tt_font_height;

}

void main (void)
{
  
  SHOW_BKG;
  DISPLAY_ON;
  empty_screen ();

  set_bkg_tiles (0,14,20,4,bkg_text_map);

  set_bkg_palette_entry(0,0,RGB(0,0,0)); 
  set_bkg_palette_entry(0,1,RGB(31,0,0)); 
  set_bkg_palette_entry(0,2,RGB(0,0,0)); 
  set_bkg_palette_entry(0,3,RGB(31,31,31));   

  tt_init ();

  while (1 == 1)
  {
    tt_empty_screen ();
    tt_set_cursor (1,1);
    tt_print_str ("Several thousand demonstrators have protested as environment ministers from G8 countries");
   
    while (joypad () == 0) ;
    while (joypad () != 0) ;
  
    tt_empty_screen ();
    tt_set_cursor (1,1);
    tt_print_str ("got down to the nitty-gritty of how to fight climate change.");
  
    while (joypad () == 0) ;
    while (joypad () != 0) ;
  }

  HIDE_BKG;
  DISPLAY_OFF;

}



